<?php
// $Id: ec_customer.checkout.inc,v 1.1.2.5 2008/11/13 03:12:44 gordon Exp $
/**
 * @file
 * Provide customer implementation for checkout
 */

/**
 * Implementation of hook_checkout_init().
 */
function ec_customer_checkout_init(&$form_state, &$form) {
  $txn =& $form_state['storage']['txn'];
  $txn->customer = ec_customer_get_customer();
  $txn->mail = ec_customer_get_email($txn->customer);
  
  ec_customer_prepare_addresses($txn);
}

/**
 * Implementation of hook_checkout_form().
 */
function ec_customer_checkout_form(&$form_state, &$form) {
  $txn =& $form_state['storage']['txn'];
  // Include address theming functions.
  if (ec_customer_get_attr($txn->customer->type, 'store_addresses', TRUE) && ($txn->shippable || variable_get('ec_customer_billing_address', 1))) {
    // Display billing (and shipping) addresses.
    if (!empty($txn->shippable)) {
      $form['ec_customer']['title'] = array( '#value' => t('Shipping and billing address') );
      $form['ec_customer']['shipping_address'] = array( '#value' => theme('ec_store_format_address', $txn, 'shipping', 'html') );
    }
    else {
      $form['ec_customer']['title'] = array( '#value' => t('Billing address') );
    }
    $form['ec_customer']['billing_address'] = array( '#value' => theme('ec_store_format_address', $txn, 'billing', 'html') );
    $form['ec_customer']['#theme'] = 'ec_customer_address';
  }
  elseif ($txn->shippable || variable_get('ec_customer_billing_address', 1)) {
    $form['ec_customer']['address'] = array(
      '#tree'  => TRUE,
      '#theme' => 'ec_customer_anon_address',
    );
    if (!empty($txn->shippable)) {
      $form['ec_customer']['address']['title'] = array( '#value' => t('Shipping and billing address') );
    }
    else {
      $form['ec_customer']['address']['title'] = array( '#value' => t('Billing address') );
    }
    $form['ec_customer']['address']['billing'] = ec_store_address_form($txn->address['billing']);
    $form['ec_customer']['address']['billing']['firstname']['#required'] = TRUE;
    $form['ec_customer']['address']['billing']['lastname']['#required'] = TRUE;
    if (!empty($txn->shippable)) {
      $form['ec_customer']['address']['billing']['#type'] = 'fieldset';
      $form['ec_customer']['address']['billing']['#collapsible'] = TRUE;
      $form['ec_customer']['address']['billing']['#title'] = t('Billing address');
      $form['ec_customer']['address']['shipping'] = ec_store_address_form($txn->address['shipping']);
      $form['ec_customer']['address']['shipping']['#type'] = 'fieldset';
      $form['ec_customer']['address']['shipping']['#collapsible'] = TRUE;
      $form['ec_customer']['address']['shipping']['#collapsed'] = TRUE;
      $form['ec_customer']['address']['shipping']['#title'] = t('Shipping address');
    }
  }
}

/**
 * Check, prepare and sanitize addresses on a transaction.
 *
 * @param $txn
 *  An ecommerce transaction or any similar object that stores address
 *  data on an address attribute.
 *
 * @return
 *  Returns FALSE if addresses could not be resolved. This will mean
 *  that the calling code needs
 *
 */
function ec_customer_prepare_addresses(&$txn, $addresses = NULL) {
  if ($addresses) {
    // The presence of new address data means we are updating the transaction.
    $txn->address = array();
  }
  
  // Assume that any $txn->address is valid.
  if (!empty($txn->address)) {
    return TRUE;
  }

  // If we lack addresses, try to gather them or return FALSE
  if (!$addresses && !isset($txn->address)) {
    if (!ec_customer_get_attr($txn->customer->type, 'store_addresses', TRUE)) {
      // We don't an address provider!
      return FALSE;
    }
    else {
      $addresses = ec_customer_get_addresses($txn->customer);
      if (!$addresses) {
        return FALSE;
      }
    }
  }
    
  // Manage the addresses returned from the address provider.
  if ($txn->shippable) {
    $txn->address['billing'] = $txn->address['shipping'] = reset($addresses);
    if (isset($txn->address['billing']->aid)) {
      $txn->billing_address = $txn->shipping_address = $txn->address['billing']->aid;
    }
  }
  elseif (variable_get('ec_customer_billing_address', 1)) {
    if (!empty($addresses)) {
      $txn->address['billing'] = reset($addresses);
      if (isset($txn->address['billing']->aid)) {
        $txn->billing_address = $txn->address['billing']->aid;
      }
    }
    else {
      $txn->billing_address = NULL;
    }
  }
  else {
    $txn->address = array();
  }    
}

/**
 * The form to set the shipping and billing address in the checkout procedure.
 */
function ec_customer_address_form($txn) {
  
  // Grab the user's addressbook
  $address = ec_customer_get_addresses($txn->customer);

  if (empty($address) && ec_customer_get_attr($txn->customer->type, 'store_addresses', TRUE)) {
    $goto = ec_customer_links($txn->customer, 'add_address', drupal_get_destination());
    call_user_func_array('drupal_goto', $goto);
  }

  // TODO: this function seems broken from here. 

  // Build address options
  $options = array();
  foreach ($address as $value) {
    $options[$value->aid] = t('@name, @street in @city',
      array(
        '@name' => ($value->fullname ? $value->fullname : "{$value->firstname} {$value->lastname}"),
        '@street' => $value->street1,
        '@city' => $value->city,
      ));
  }
  if (!empty($txn->shippable)) {
    $form['shipping_address'] = array(
      '#type'           => 'radios',
      '#title'          => t('Shipping to'),
      '#default_value'  => $txn->shipping_address,
      '#options'        => $options,
      '#description'    => t('Please choose where you would like the items to be delivered.') .
        (ec_customer_get_attr($txn->customer, 'add_address', TRUE) ?
        ' '. t('You can also <a href="!add_address">add a new address</a>.', array(
        '!add_address' => call_user_func_array('url',
        ec_customer_links($txn->customer, 'add_address', drupal_get_destination())))) : '')
    );
  }
  $form['billing_address'] = array(
    '#type'           => 'radios',
    '#title'          => t('Billing to'),
    '#default_value'  => $txn->billing_address,
    '#options'        => $options,
    '#description'    => t('Please choose where you would like the bill to be delivered.') .
      (ec_customer_get_attr($txn->customer, 'add_address', TRUE) ?
      ' '. t('You can also <a href="!add_address">add a new address</a>.',
      array('!add_address' => call_user_func_array('url',
      ec_customer_links($txn->customer, 'add_address', drupal_get_destination())))) : '')
  );
  #$form['address']['#tree'] = TRUE;
  return $form;
}

/**
 * Implementation of hook_checkout_submit().
 */
function ec_customer_checkout_submit(&$form_state, &$form) {
  $txn =& $form_state['storage']['txn'];
  if (!empty($form_state['values']['address']['billing'])) {
    $txn->address['billing'] = (object)$form_state['values']['address']['billing'];
  }
  if (!empty($form_state['values']['address']['shipping'])) {
    $txn->address['shipping'] = (object)$form_state['values']['address']['shipping'];
  }
}

/**
 * Implementation of hook_checkout_init().
 */
function ec_customer_email_checkout_form(&$form_state, &$form) {
  $txn =& $form_state['storage']['txn'];
  $form['ec_customer_email'] = array(
    '#theme' => 'ec_mail_checkout_form'
  );
  if ($txn->customer->type == 'anonymous') {
    $form['ec_customer_email']['mail'] = array(
      '#type' => 'textfield',
      '#title' => t('Email'),
      '#default_value' => $txn->mail,
      '#validate' => array('ec_anon_validate_mail' => array()),
      '#required' => TRUE,
    );
  }
  else {
    $form['ec_customer_email']['mail'] = array(
      '#value' => $txn->mail ? $txn->mail : ec_customer_get_email($txn->ecid),
    );
    $form['ec_customer_email']['mail']['#access'] = variable_get('ec__display_email', 1);
  }
}

/**
 * Implementation of hook_checkout_submit().
 */
function ec_customer_email_checkout_submit(&$form_state, &$form) {
  $txn =& $form_state['storage']['txn'];
  if (!empty($form_state['values']['mail'])) {
    $txn->mail = $form_state['values']['mail'];
  }
}

